@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.24 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2014, Mark Calabretta
+  WCSLIB 5.14 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2016, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,7 +22,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: tcel2.c,v 4.24 2014/09/18 15:25:01 mcalabre Exp $
+  $Id: tcel2.c,v 5.14 2016/02/07 10:49:32 mcalabre Exp $
 *=============================================================================
 *
 * tcel2 thoroughly tests the WCSLIB celestial coordinate transformation
@@ -525,7 +525,7 @@ int main()
 
           } else {
             printf("Next, skip, break, continue, exit [%c]: ", ctrl);
-            fgets(answer, 16, stdin);
+            if (fgets(answer, 16, stdin) == 0) answer[0] = 0;
 
             if (strchr("bBcCeEiIjJnNpPqQsS", (int)answer[0]) != 0) {
               ctrl = answer[0];
