@@ -1,7 +1,7 @@
 *=======================================================================
 *
-* WCSLIB 4.24 - an implementation of the FITS WCS standard.
-* Copyright (C) 1995-2014, Mark Calabretta
+* WCSLIB 5.14 - an implementation of the FITS WCS standard.
+* Copyright (C) 1995-2016, Mark Calabretta
 *
 * This file is part of WCSLIB.
 *
@@ -22,12 +22,12 @@
 *
 * Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
 * http://www.atnf.csiro.au/people/Mark.Calabretta
-* $Id: lin_data.f,v 4.24 2014/09/18 15:25:01 mcalabre Exp $
+* $Id: lin_data.f,v 5.14 2016/02/07 10:49:32 mcalabre Exp $
 *=======================================================================
 
       BLOCK DATA LIN_BLOCK_DATA
 
-      CHARACTER LIN_ERRMSG(0:3)*80
+      CHARACTER LIN_ERRMSG(0:6)*80
 
       COMMON /LIN_DATA/ LIN_ERRMSG
 
@@ -35,6 +35,9 @@
      :  'Success',
      :  'Null linprm pointer passed',
      :  'Memory allocation failed',
-     :  'PCi_ja matrix is singular'/
+     :  'PCi_ja matrix is singular',
+     :  'Failed to initialize distortion functions',
+     :  'Distort error',
+     :  'De-distort error'/
 
       END
