@@ -1,7 +1,7 @@
 /*============================================================================
 
-  WCSLIB 4.23 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2014, Mark Calabretta
+  WCSLIB 5.14 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2016, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -22,7 +22,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: HPXcvt.c,v 4.23 2014/05/13 05:50:54 mcalabre Exp $
+  $Id: HPXcvt.c,v 5.14 2016/02/07 10:49:33 mcalabre Exp $
 *=============================================================================
 *
 * HPXcvt reorganises HEALPix data into a 2-D FITS image.  Refer to the usage
@@ -855,7 +855,7 @@ int HPXhdr(fitsfile *fptr, struct healpix *hpxdat)
 
   if (hpxdat->layout) {
     lonpole = 180.0f;
-    sprintf(comment, "[deg] Native longitude of the celestial pole", descr1);
+    sprintf(comment, "[deg] Native longitude of the celestial pole");
     fits_write_key(fptr, TFLOAT, "LONPOLE", &lonpole, comment, &status);
   }
 
